from flask import Flask, request, jsonify
from flask_cors import CORS
from werkzeug.utils import secure_filename
from crewai import Agent, Task, Crew, Process
from my_tools import pdfScrapingTool
import os, json, psycopg2

# Initialize Flask app
app = Flask(__name__)
CORS(app)  # Enable cross-origin requests

# Ensure OpenAI API Key is set
os.environ["OPENAI_API_KEY"] = "sk-proj-xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxqJnl"

UPLOAD_FOLDER = 'uploads'
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER


# Initialize the tool allowing for any PDF content search if the path is provided during execution



# PostgreSQL connection
DB_CONFIG = {
    'dbname': 'candidates',
    'user': 'user_db',
    'password': 'password',  # Set your PostgreSQL password
    'host': 'localhost',
    'port': '5432'
}
conn = psycopg2.connect(**DB_CONFIG)


# Ensure upload folder exists
if not os.path.exists(UPLOAD_FOLDER):
    os.makedirs(UPLOAD_FOLDER)


@app.route('/upload', methods=['POST'])
def upload_cv():
    """Handles CV Upload and Extraction"""
    if 'files' not in request.files:
        return jsonify({'message': 'No file uploaded'}), 400

    files = request.files.getlist('files')
    for file in files:
        filename = secure_filename(file.filename)
        filepath = os.path.join(app.config['UPLOAD_FOLDER'], filename)
        file.save(filepath)
        process_cv(filepath)  # Process the uploaded CV

    return jsonify({'message': 'CVs uploaded and processed successfully'})


def process_cv(filepath):
    # Initialize the tool with a specific PDF path for exclusive search within that document
    
    
    """Extracts information from CV using CrewAI"""
    
    agent = Agent(
        role='CV Processor',
        goal='Extract candidate details from CVs',
        verbose=True,
        memory=True,
        backstory="Expert in parsing and analyzing resumes.",
        llm='gpt-4o-mini',
        tools=[pdfScrapingTool]
    )

    task = Task(
        description=f"Extract name, email and skills of the candidate from the PDF cv located in path '{filepath}' and return structured JSON.",
        expected_output="""A structured JSON without comments with respect to the bellow strucure: 
                {{
            "name": string,             // the full name of the candidat
            "email": string,           // email of the candidat
            "skills": string        // list of skills of the candidat 
             
            
            }}
        
        """,
        tools=[pdfScrapingTool],
        agent=agent
    )

    # Define the Crew (Attach Agents and Tasks)
    crew = Crew(
        agents=[agent],
        tasks=[task],
        process=Process.sequential  # Tasks will be executed one after another
    )

    # Execute the Crew (Run the Task)
    result = crew.kickoff()

    # Print the extracted candidate information
    print("Extracted Candidate Information:")
    save_to_db(json.loads(str(result)))


def save_to_db(json_data: dict):
    """Saves extracted data into PostgreSQL"""
    query = "INSERT INTO candidates (name, email, skills) VALUES (%s, %s, %s)"
    cursor = conn.cursor()
    cursor.execute(query, (json_data['name'], json_data['email'], json_data['skills']))
    conn.commit()
    cursor.close()

def extract_from_db() -> str:
    """Saves extracted data into PostgreSQL"""
    query = "select name, email, skills from candidates "
    cursor = conn.cursor()
    cursor.execute(query)
    rows = cursor.fetchall()

    # Get column names
    col_names = [desc[0] for desc in cursor.description]
    if not rows:  # Check if the result is empty
        json_result = json.dumps({"message": "No data found"})
    else:
        # Transform result into JSON format
        json_result = json.dumps([dict(zip(col_names, row)) for row in rows], indent=4)

    # Print or use the JSON result
    print(json_result) 
    cursor.close() 
    return json_result    

@app.route('/chat', methods=['POST'])
def chat():
    """Handles chatbot queries"""
    message = request.json.get('message', '')
    db_candidates = extract_from_db();
    agent = Agent(
        role="Chatbot",
        goal="Answer HR-related questions based on candidates data.",
        verbose=True,
        memory=True,
        backstory="Specialist in answering queries about candidates and their skills.",
        llm='gpt-4o-mini'
    )

    task = Task(
        description=f"Answer the following query: {message} based in the following data extracted from database in json format: '{db_candidates}'",
        expected_output="A detailed response based on the quesry. select only candidates that fill qith the query",
        agent=agent
    )
    # Define the Crew (Attach Agents and Tasks)
    crew = Crew(
        agents=[agent],
        tasks=[task],
        process=Process.sequential  # Tasks will be executed one after another
    )

    # Execute the Crew (Run the Task)
    result = crew.kickoff()

    # Print the extracted candidate information
    print("response to question:")
    print(result)
    
    return jsonify({'response': str(result)})


if __name__ == '__main__':
    app.run(debug=True)